/* 
   LINWLOAD ver 1.0a
   (c) Wojciech M. Zabolotny (wzab@ise.pw.edu.pl) 15.08.2000
   This file is published under the Gnu Public License version 2

   This file contains the routines necessary to read the contents of 
   MPLAB's HEX file.
   The routine HexRead reads the file and fills two arrays:
   1) The Mem array contains the read data
   2) The Used array contains '0's at the addresses not defined in the
      hex file and '1's at the addresses defined in the hex file.
*/
#include "linwload.h"

/* Function wzstrlen returns length of C-string, pointed by 'n', without
   the ending 0 char and 0xd or 0xa chars.
*/
int wzstrlen(char * n)
{
  int i;
  for(i=0;n[i]!=0 && n[i]!=0xd && n[i]!=0xa;i++) {};
  return i;
}

/* Function asc2hex returns hexadecimal value of two adjacent ASCII chars
   pointed by 'n' */

unsigned int asc2hex(char * n)
{
  int rn=0;
  sscanf(n,"%2x",&rn);
  return rn;
}

/* Function asc4hex returns hexadecimal value of four adjacent ASCII chars
   pointed by 'n' */

unsigned int asc4hex(char * n)
{
  int rn=0;
  sscanf(n,"%4x",&rn);
  return rn;
}

unsigned short int Mem[MEM_LEN];
char Used[MEM_LEN];

int HexRead(char * hexfn)
{
#define INLINE_LEN 256
  char InputLine[INLINE_LEN];
  char finish=0;
  int linlen=0;
  int wlen=0;
  int addr=0;
  int data=0;
  int d1,d2,i;
  int chksum;
  FILE * hexfile;
  memset(Mem,0,sizeof(Mem));
  memset(Used,0,sizeof(Used));
  hexfile = fopen(hexfn,"rt");
  if(!hexfile)
    {
      fprintf(stderr,"I can't open hexfile: %s\n",hexfn);
      exit(-1);
    }
  /* Now we are reading the data */
  while(!feof(hexfile) && !finish) 
    {
      fscanf(hexfile,"%s",InputLine); //Read line from hex file
      //printf("%s\n",InputLine);fflush(stdout);
      if((linlen=wzstrlen(InputLine))!=0) {
	wlen    = asc2hex(&InputLine[1]);
	//printf("wlen=%d\n",wlen);
	addr   = asc4hex(&InputLine[3]);
	if (addr & 1)
	  {
	    printf("Error, the address may not be odd!\n");
	    exit(-1);
	  }
	addr/=2;
	finish = asc2hex(&InputLine[7]);
	chksum    = 0;
	//First check the checksum
	for(i=1;i<linlen;i+=2)
	  {
	    chksum = (chksum + asc2hex(&InputLine[i])) & 0xff;
	  }
	if (chksum)
	  {
	    fprintf(stderr,"Wrong chksum in HEX file!\n"
		    "Was %x\n"
			"Line: %s\n",
		    chksum,InputLine);
	    //exit(-1);
	  }
	
	//Repeat sending line, until EV51 confirms
	for(i=0;i<wlen/2;i++)
	  {
	    int line_ind = 9+4*i;
	    if (line_ind+4>linlen) 
	      {
		fprintf(stderr,"HEX file corrupted?\n");
		exit(-1);
	      }
	    d1 = asc2hex(&InputLine[line_ind]);
	    d2 = asc2hex(&InputLine[line_ind+2]);
	    //printf("%x,%x\n",d1,d2);
	    Mem[addr+i] = d1+256*d2;
	    Used[addr+i] = 1;
	  }
      }
    }
  fclose(hexfile);
}
