/*
   LINWLOAD ver 1.0a
   (c) Wojciech M. Zabolotny (wzab@ise.pw.edu.pl) 15.08.2000
   This file is published under the Gnu Public License version 2

   This file implements the routines necessary for RS232 interface
*/

#include "linwload.h"
char * ComDevice = "/dev/ttyS3";
int Com_fd = -1;
tcflag_t BaudRate=B19200;

//The rsinit opens the com port and resets the target 
int rsinit(void)
{
  //First use setserial to set the speed
  char buf[256];
  struct termios newtio;
  Com_fd = open(ComDevice, O_RDWR | O_NOCTTY | O_NONBLOCK);
  if (Com_fd <0) {perror(ComDevice); exit(-1); }
  bzero(&newtio,sizeof(newtio));
  newtio.c_cflag = BaudRate | CS8 | CLOCAL | CREAD;
  //  newtio.c_iflag = IGNPAR | ICRNL;
  newtio.c_iflag = 0;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
  newtio.c_cc[VMIN]=1;
  newtio.c_cc[VTIME]=0;
  tcflush(Com_fd, TCIFLUSH);
  tcsetattr(Com_fd,TCSANOW,&newtio);
  //Now let's switch on DTR line to reset the PIC
  {
    int flags;
    if(-1==ioctl(Com_fd,TIOCMGET,&flags)){perror(ComDevice); exit(-1); } 
    flags |= TIOCM_DTR;
    if(-1==ioctl(Com_fd,TIOCMSET,&flags)){perror(ComDevice); exit(-1); }
  }
  sleep(2);
  //Now let's switch off DTR line
  {
    int flags;
    if(-1==ioctl(Com_fd,TIOCMGET,&flags)){perror(ComDevice); exit(-1); } 
    flags &= ~TIOCM_DTR;
    if(-1==ioctl(Com_fd,TIOCMSET,&flags)){perror(ComDevice); exit(-1); }
  }
  sleep(1);
}

//Returns the byte read from RS, or -1 if there is no char available
int rsget(void)
{
  unsigned char c;
  if(read(Com_fd,&c,1)<1) return -1;
  return (int) c;
}

//Sends the byte through RS
void rsput(int data)
{
  unsigned char c = (unsigned char) data;
  if(write(Com_fd,&c,1)!=1) {perror(ComDevice); exit(1);}
}

//Sends the string through RS
void rssend(char * str)
{
  while (*str)
    {
      rsput((int)*str);
      str++;
    }
}

//Closes the RS232
void rsleave(void)
{
  close(Com_fd);
  Com_fd=-1;
}

//Restores the normal terminal settings
void restoreterm(void)
{
  system("stty icanon echo");
}    

//Checks if the key has been pressed
int kbhit()
{
  fd_set rfds;
  struct timeval tv;
  int retval;
  /* Watch stdin (fd 0) to see when it has input. */
  FD_ZERO(&rfds);
  FD_SET(0, &rfds);
  /* Wait up to five seconds. */
  tv.tv_sec = 0;
  tv.tv_usec = 0;
  retval = select(1, &rfds, NULL, NULL, &tv);
  return retval;
}

/* This routine allows program to work as a terminal
   chars generated by pressed keys are sent to EV51 board
   chars received from EV51, are displayed on host's screen */

//This procedure implements the simple terminal
void terminal(void)
{
  int b;
  printf("Terminal mode entered, press '~' to exit\n");
  for(;;) 
    {
      if(kbhit()) rsput(b=fgetc(stdin)); // Send entered char
      if(b=='~') break; // Char '~' ends program
      if((b=rsget())!=-1)
	{
	  fputc(b&255,stdout); // Display received char
	  fflush(stdout);
	}
    }
}


