/* 
   LINWLOAD ver 1.03a
   (c) Wojciech M. Zabolotny (wzab@ise.pw.edu.pl) 17.09.2000
   This file is published under the Gnu Public License version 2

   This file contains routines implementing the WLoader protocol
*/
#include "linwload.h"

//The routine below sends the byte, receives the echo, and then receives 
//the answer (waiting for it given amount of seconds)
int WGetWait(int sec)
{
  fd_set rfds;
  struct timeval tv;
  int retval;
  /* Watch stdin (fd 0) to see when it has input. */
  FD_ZERO(&rfds);
  FD_SET(Com_fd, &rfds);
  /* Wait up to two seconds. */
  tv.tv_sec = sec;
  tv.tv_usec = 0;
  retval = select(Com_fd+1, &rfds, NULL, NULL, &tv);
  if(retval) return rsget();
  return -1;
}

//This routine sends a byte, verifies the "hardware echo", and
//returns the WLoader's response
int WPut(char c)
{
  int ret;
  if(rsget()!=-1) 
    {
      fprintf(stderr,"Communication error, received char before sending anything\n");
      exit(-1);
    }
  rsput(c);
  if(WGetWait(2)!=c)
    {
      fprintf(stderr,"Communication error, sent character not echoed correctly\n");
      exit(-1);
    }
  ret=WGetWait(2);
  if(ret==-1)
    {
      fprintf(stderr,"Communication error, response not received\n");
      exit(-1);
    }
  return ret;
}

// The procedure below sends the given string and puts in the receive
//buffer  the received answer 
void RSSendRcv(char * sent, char * rcvd, int n)
{
  int i,l;
  l=strlen(sent);
  sent[l]=0;
  rcvd[l]=0;
  for (i=0;i<l;i++)
    {
      rcvd[i]=WPut(sent[i]);
    }
}

// The procedure below implements the T command
void WloadT()
{
  char msg[6]="TNNNN";
  char ans[6];
  char expd[]="TWLdr";
  int i;
  RSSendRcv(msg,ans,strlen(msg));
  if(strcmp(ans,expd)) 
    {
      fprintf(stderr,"Wrong loader type! expected: %s, received: %s\n",
	      expd,ans);
      exit(-1);
    }
}

//The procedure below implements the V command
void WLoadV()
{
  char msg[6]="VNNNN";
  char ans[6];
  char expd[]="V0103";
  RSSendRcv(msg,ans,strlen(msg));
  if(strcmp(ans,expd)) 
    {
      fprintf(stderr,"Wrong loader version! expected: %s, received: %s\n",
	      expd,ans);
      exit(-1);
    }
}

//The procedure below implements the CX command
void WLoadCX()
{
  char msg[3]="CX";
  char ans[3];
  char expd[]="CX";
  RSSendRcv(msg,ans,strlen(msg));
  if(strcmp(ans,expd)) 
    {
      fprintf(stderr,"Wrong answer! expected: %s, received: %s\n",
	      expd,ans);
      exit(-1);
    }
}

//The procedure below implements the DX command
void WLoadDX()
{
  char msg[3]="DX";
  char ans[3];
  char expd[]="DX";
  RSSendRcv(msg,ans,strlen(msg));
  if(strcmp(ans,expd)) 
    {
      fprintf(stderr,"Wrong answer! expected: %s, received: %s\n",
	      expd,ans);
      exit(-1);
    }
}

//The procedure below implements the FX command
void WLoadFX()
{
  char msg[3]="FX";
  char ans[3];
  char expd[]="FX";
  RSSendRcv(msg,ans,strlen(msg));
  if(strcmp(ans,expd)) 
    {
      fprintf(stderr,"Wrong answer! expected: %s, received: %s\n",
	      expd,ans);
      exit(-1);
    }
}

//The procedure below implements the I command
void WLoadI()
{
  char msg[2]="I";
  char ans[2];
  char expd[]="I";
  RSSendRcv(msg,ans,strlen(msg));
  if(strcmp(ans,expd)) 
    {
      fprintf(stderr,"Wrong answer! expected: %s, received: %s\n",
	      expd,ans);
      exit(-1);
    }
}

//The procedure below implements the R command (returning the read value)
int WLoadR()
{
  char msg[6]="RNNNN";
  char ans[6];
  int res;
  RSSendRcv(msg,ans,strlen(msg));
  if(ans[0]!='R') 
    {
      fprintf(stderr,"Wrong answer! expected: R, received: %c\n",
	      ans[0]);
      exit(-1);
    }
  sscanf(&ans[1],"%x",&res);
  return res;
}

//The procedure below implements the L command
void WLoadL(int val)
{
  char msg[20]; //Longer to avoid problems for val > 0xffff 
  //(however such values should never happen
  char ans[20];
  sprintf(msg,"%04XL",val);
  RSSendRcv(msg,ans,strlen(msg));
  if(ans[4]=='?')
    {
      fprintf(stderr,"Programming failed!");
      exit(-1);
    }
  if(strcmp(msg,ans)) 
    {
      fprintf(stderr,"Wrong answer! expected: %s, received: %s\n",
	      msg,ans);
      exit(-1);
    }
}

void Prog(void)
{
  int i;
  if(Com_fd != -1) rsleave();
  //First open the RS232C and reset target
  rsinit();
  //Check the type and version of the loader
  WloadT();
  WLoadV();
  //Program the code memory
  WLoadCX();
  for(i=0;i<=CODE_MAX;i++)
    {
      if(Used[i])
	{
	  WLoadL(Mem[i]);
	}
      WLoadI();
      if(!(i%0x10)) printf("%x OK\n",i);
    }  
  //Verify the data EEPROM memory
  WLoadDX();
  for(i=DATA_MIN;i<=DATA_MAX;i++)
    {
      if(Used[i])
	{
	  WLoadL(Mem[i]);
	}
      WLoadI();
      if(!(i%0x10)) printf("%x OK\n",i);
    }  
  //Verify the fuses memory
  WLoadFX();
  for(i=FUSE_MIN;i<=FUSE_MAX;i++)
    {
      if(Used[i])
	{
	  WLoadL(Mem[i]);
	}
      WLoadI();
      if(!(i%0x10)) printf("%x OK\n",i);
    }  
}

void Verify(void)
{
  int i;
  //First open the RS232C and reset target
  if(Com_fd != -1) rsleave();
  rsinit();
  //Check the type and version of the loader
  WloadT();
  WLoadV();
  //Verify the code memory
  WLoadCX();
  for(i=0;i<=CODE_MAX;i++)
    {
      if(Used[i])
	{
	  int res=WLoadR();
	  if(res!=Mem[i])
	    {
	      fprintf(stderr,"Verification error: addr=%x, file=%x, PIC=%x\n",
		      i,Mem[i],res);
	      exit(-1);
	    }
	}
      WLoadI();
      if(!(i%0x10)) printf("%x OK\n",i);
    }  
  //Verify the data EEPROM memory
  WLoadDX();
  for(i=DATA_MIN;i<=DATA_MAX;i++)
    {
      if(Used[i])
	{
	  int res=WLoadR();
	  if(res!=Mem[i])
	    {
	      fprintf(stderr,"Verification error: addr=%x, file=%x, PIC=%x\n",
		      i,Mem[i],res);
	      exit(-1);
	    }
	}
      WLoadI();
      if(!(i%0x10)) printf("%x OK\n",i);
    }  
  //Verify the fuses memory
  WLoadFX();
  for(i=FUSE_MIN;i<=FUSE_MAX;i++)
    {
      if(Used[i])
	{
	  int res=WLoadR();
	  if(res!=Mem[i])
	    {
	      fprintf(stderr,"Verification error: addr=%x, file=%x, PIC=%x\n",
		      i,Mem[i],res);
	      exit(-1);
	    }
	}
      WLoadI();
      if(!(i%0x10)) printf("%x OK\n",i);
    }  
}

void Go(void)
{
  char msg[5]="0000";
  char ans[5];
  if(Com_fd==-1) rsinit();
  RSSendRcv(msg,ans,strlen(msg));
  rsput('G'); //It must be done that way, because even the "hardware echo"
  // after the "go" command often gets distorted... :-(.
}
